==> Parallel-Workers-0.0.5.tar.gz: Parallel-Workers-0.0.5/lib/Parallel/Workers/Transaction.pm <==
package Parallel::Workers::Transaction;

use warnings;
use strict;
use Carp;
use Data::Dumper;
use threads::shared;
use Parallel::Workers::Shared;

our (@ISA, @EXPORT, @EXPORT_OK);
@ISA = qw(Exporter);

@EXPORT = qw(TRANSACTION_CONT TRANSACTION_TERM);
@EXPORT_OK = ();

use constant TRANSACTION_CONT=>"CONT";
use constant TRANSACTION_TERM=>"TERM";


our $locker:shared=0;
# Module implementation here
# init the transaction for n workers 
# pre=>CONT|TERM, do =>CONT|TERM, post=>CONT|TERM
sub new {
    my $class:shared = shift;
    my %args = @_;

    my $this={};
    my $locker:shared=0;
    shared_hash_set($this,'workers',{});  
    shared_hash_set($this,'count',0);  
    shared_hash_set($this,'pre',(defined ($args{pre}))?$args{pre}:TRANSACTION_CONT );  
    shared_hash_set($this,'do',(defined ($args{do}))?$args{do}:TRANSACTION_CONT);  
    shared_hash_set($this,'post',(defined ($args{post}))?$args{post}:TRANSACTION_CONT );  
    shared_hash_set($this,'enable',(defined ($args{enable}))?$args{enable}:0);  
    shared_hash_set($this,'status','end');  
    
    bless $this, $class;
    return $this;
}

sub commit{
  my $this=shift;
  shared_hash_set($this,'workers',{});  
  $this->{count}=0;
  $this->{status}="init";
  return $this;
}

sub start{
  my $this=shift;
  my ($count)=@_;
  $this->{count}=$count;
  $this->{status}="start";
  return $this;
}

sub end{
  my $this=shift;
  $this->{status}="end";
  return $this;
}

sub status{
  my $this=shift;
  return $this->{status};
}



sub put{
  my $this=shift;
  my ($tid, $result)=@_;
  shared_hash_set($this->{workers},$tid,$result);
  
 lock($locker);
  print "put new result for tid=$tid\n";
# signal that work is done
  if((values %{$this->{workers}})>=$this->{count}){
    print "signal that job  is terminated...";
    cond_broadcast($locker) ;
    print "done.\n";
  }
  return $this;
}

# block the current thread until other have finished
sub continue{
  my $this=shift;
  my ($cmd)=@_;  #pre,do,post
  return TRANSACTION_CONT unless $this->{enable};
  
 lock($locker);
  if((values %{$this->{workers}})<$this->{count}){
    print "waiting for all results (current ".(values %{$this->{workers}}).")\n";
    cond_wait($locker);
    print "all results are here!\n";
  }
  
#check if work is well done !!
  print "check transaction end (".(keys %{$this->{workers}}).")\n";
  foreach my $tid (keys %{$this->{workers}}){
#    print "return error :".$this->{$cmd}.", for tid=".$this->{workers}->{$tid}."\n";
    $this->{status} ="error" && return $this->{$cmd} if ($this->{workers}->{$tid} ne "0");
  }
  return TRANSACTION_CONT;
}


1; # Magic true value required at end of module
__END__

=head1 NAME

Parallel::Workers::Transaction - [One line description of module's purpose here]


=head1 VERSION

This document describes Parallel::Workers::Transaction version 0.0.1


=head1 SYNOPSIS

    use Parallel::Workers::Transaction;

=for author to fill in:
    Brief code example(s) here showing commonest usage(s).
    This section will be as far as many users bother reading
    so make it as educational and exeplary as possible.
  
  
=head1 DESCRIPTION

=for author to fill in:
    Write a full description of the module and its features here.
    Use subsections (=head2, =head3) as appropriate.


=head1 INTERFACE 

=for author to fill in:
    Write a separate section listing the public components of the modules
    interface. These normally consist of either subroutines that may be
    exported, or methods that may be called on objects belonging to the
    classes provided by the module.


=head1 DIAGNOSTICS

=for author to fill in:
    List every single error and warning message that the module can
    generate (even the ones that will "never happen"), with a full
    explanation of each problem, one or more likely causes, and any
    suggested remedies.

=over

=item C<< Error message here, perhaps with %s placeholders >>

[Description of error here]

=item C<< Another error message here >>

[Description of error here]

[Et cetera, et cetera]

=back


=head1 CONFIGURATION AND ENVIRONMENT

=for author to fill in:
    A full explanation of any configuration system(s) used by the
    module, including the names and locations of any configuration
    files, and the meaning of any environment variables or properties
    that can be set. These descriptions must also include details of any
    configuration language used.
  
Parallel::Workers::Transaction requires no configuration files or environment variables.


=head1 DEPENDENCIES

=for author to fill in:
    A list of all the other modules that this module relies upon,
    including any restrictions on versions, and an indication whether
    the module is part of the standard Perl distribution, part of the
    module's distribution, or must be installed separately. ]

None.


=head1 INCOMPATIBILITIES

=for author to fill in:
    A list of any modules that this module cannot be used in conjunction
    with. This may be due to name conflicts in the interface, or
    competition for system or program resources, or due to internal
    limitations of Perl (for example, many modules that use source code
    filters are mutually incompatible).

None reported.


=head1 BUGS AND LIMITATIONS

=for author to fill in:
    A list of known problems with the module, together with some
    indication Whether they are likely to be fixed in an upcoming
    release. Also a list of restrictions on the features the module
    does provide: data types that cannot be handled, performance issues
    and the circumstances in which they may arise, practical
    limitations on the size of data sets, special cases that are not
    (yet) handled, etc.

No bugs have been reported.

Please report any bugs or feature requests to
C<bug-parallel-jobs-backend@rt.cpan.org>, or through the web interface at
L<http://rt.cpan.org>.


=head1 AUTHOR

Olivier Evalet  C<< <evaleto@gelux.ch> >>


=head1 LICENCE AND COPYRIGHT

Copyright (c) 2006, Olivier Evalet C<< <evaleto@gelux.ch> >>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.

==> Parallel-Workers-0.0.7.tar.gz: Parallel-Workers-0.0.7/lib/Parallel/Workers/Transaction.pm <==
package Parallel::Workers::Transaction;

use warnings;
use strict;
use Carp;
use Data::Dumper;
use threads::shared;
use Parallel::Workers::Shared;

our (@ISA, @EXPORT, @EXPORT_OK);
@ISA = qw(Exporter);

@EXPORT = qw(TRANSACTION_CONT TRANSACTION_TERM);
@EXPORT_OK = ();

use constant TRANSACTION_CONT=>"CONT";
use constant TRANSACTION_TERM=>"TERM";


# Module implementation here
# init the transaction for n workers 
# error=>TRANSACTION_CONT, enable=>1, type => SCALAR,check => 0, regex=> undef
sub new {
    my $class:shared = shift;
    my %args = @_;

    my $this={ 
              error =>(defined $args{error})?$args{error}:TRANSACTION_CONT,
              enable=>(defined $args{enable})?$args{enable}:1,
              type  =>(defined $args{type})?$args{type}:"SCALAR",
              check =>$args{check},
              regex =>$args{regex}
             };
              
          
    bless $this, $class;
    return $this;
}


sub check{
  my $this=shift;
  my ($tid, $result)=@_;
  
  return TRANSACTION_CONT unless $this->{enable};
  
  return $this->{error} unless ($this->{type} eq ref(\$result));
  
  if (defined $this->{regex}){
    my $reg=qr/$this->{regex}/;
#    my $reg=qr/$this->{regex}/;
#    print "INFO:regex detected".$this->{regex}."\n" unless defined $result || $result=~ $reg ;
    return $this->{error} unless defined $result ;
    return $this->{error} unless $result=~ $this->{regex} ;
}
   
  if (defined $this->{check}){
   return $this->{error} unless $result=$this->{check};
  } 
  
  return TRANSACTION_CONT;
}



1; # Magic true value required at end of module
__END__

=head1 NAME

Parallel::Workers::Transaction - [One line description of module's purpose here]


=head1 VERSION

This document describes Parallel::Workers::Transaction version 0.0.1


=head1 SYNOPSIS

    use Parallel::Workers::Transaction;

=for author to fill in:
    Brief code example(s) here showing commonest usage(s).
    This section will be as far as many users bother reading
    so make it as educational and exeplary as possible.
  
  
=head1 DESCRIPTION

=for author to fill in:
    Write a full description of the module and its features here.
    Use subsections (=head2, =head3) as appropriate.


=head1 INTERFACE 

=for author to fill in:
    Write a separate section listing the public components of the modules
    interface. These normally consist of either subroutines that may be
    exported, or methods that may be called on objects belonging to the
    classes provided by the module.


=head1 DIAGNOSTICS

=for author to fill in:
    List every single error and warning message that the module can
    generate (even the ones that will "never happen"), with a full
    explanation of each problem, one or more likely causes, and any
    suggested remedies.

=over

=item C<< Error message here, perhaps with %s placeholders >>

[Description of error here]

=item C<< Another error message here >>

[Description of error here]

[Et cetera, et cetera]

=back


=head1 CONFIGURATION AND ENVIRONMENT

=for author to fill in:
    A full explanation of any configuration system(s) used by the
    module, including the names and locations of any configuration
    files, and the meaning of any environment variables or properties
    that can be set. These descriptions must also include details of any
    configuration language used.
  
Parallel::Workers::Transaction requires no configuration files or environment variables.


=head1 DEPENDENCIES

=for author to fill in:
    A list of all the other modules that this module relies upon,
    including any restrictions on versions, and an indication whether
    the module is part of the standard Perl distribution, part of the
    module's distribution, or must be installed separately. ]

None.


=head1 INCOMPATIBILITIES

=for author to fill in:
    A list of any modules that this module cannot be used in conjunction
    with. This may be due to name conflicts in the interface, or
    competition for system or program resources, or due to internal
    limitations of Perl (for example, many modules that use source code
    filters are mutually incompatible).

None reported.


=head1 BUGS AND LIMITATIONS

=for author to fill in:
    A list of known problems with the module, together with some
    indication Whether they are likely to be fixed in an upcoming
    release. Also a list of restrictions on the features the module
    does provide: data types that cannot be handled, performance issues
    and the circumstances in which they may arise, practical
    limitations on the size of data sets, special cases that are not
    (yet) handled, etc.

No bugs have been reported.

Please report any bugs or feature requests to
C<bug-parallel-jobs-backend@rt.cpan.org>, or through the web interface at
L<http://rt.cpan.org>.


=head1 AUTHOR

Olivier Evalet  C<< <evaleto@gelux.ch> >>


=head1 LICENCE AND COPYRIGHT

Copyright (c) 2006, Olivier Evalet C<< <evaleto@gelux.ch> >>. All rights reserved.

This module is free software; you can redistribute it and/or
modify it under the same terms as Perl itself. See L<perlartistic>.


=head1 DISCLAIMER OF WARRANTY

BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
NECESSARY SERVICING, REPAIR, OR CORRECTION.

IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGES.
